package Outputs;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import org.javatuples.Pair;

import TemporalNetworkManipulation.Community;
import TemporalNetworkManipulation.MyDate;
import TemporalNetworkManipulation.MyDateManager;
import TemporalNetworkManipulation.Network;
import TemporalNetworkManipulation.Node;



/**this class is to print static communities. It prints only the communities alive at the end of the process.
 * the communities are printed in a way allowing overlap : each line correspond to a community. 
 * Each line contains all the nodes of the community. Standard format as used by several static algorithms
 *
 * @author remy cazabet
 *
 */
public class OutputNetworkOfOperations implements OutputHandlerInterface{

	
	private String separator = "	" ; //tab
	PrintWriter fileout=null;
	Network theNetwork;
	
	String previousPeriod;

	String currentPeriod;
	
	int idCom = 0;
	int minimumCommunitySizeForOutput = 5;
	HashMap<Community,Integer> existingCommunities = new HashMap<>();
	Set<Community> currentCommunities = new HashSet<>();


	ArrayList<Community> tempNewCommunities = new ArrayList<Community>();
	
	ArrayList<Pair<Community,Community>> tempMerge = new ArrayList<>();
	
	Set<Community> alreadyDisplayedCommuniteis = new HashSet<>();
	
	public OutputNetworkOfOperations(MyDateManager dateManager, Network theNetwork) {
		this.theNetwork = theNetwork;
	}

	public void initialise(String outputFile){
		
		try {
			fileout = new PrintWriter(new BufferedWriter (new FileWriter(outputFile+".edges")));
		} catch (IOException e) {
			System.out.println("problem to write in :  "+ outputFile);
			e.printStackTrace();
		}
	}
	
	public void terminate(){
		this.previousPeriod = this.currentPeriod;
		this.currentPeriod="END";
		
		finalizePeriod();
		fileout.close();
	}
	
	public void printLine(String line){

		fileout.println(line);
	}
	
	
	
	
	public void handleGrowth(Node n, Community c){
		
	}
	
	public void handleGrowth(Community initialCommunity, Community resultingCommunity){
		
	}

	public void handleBirth(Community c) {
		tempNewCommunities.add(c);

		
	}

	public void handleDeath(Community c) {
		tempNewCommunities.remove(c);
		currentCommunities.remove(c);
		//existingCommunities.remove(c);
		
	}

	
	public void handleContraction(Node n, Community c) {
	}
	
	public void handleNewEdge(Node n, Node n2) {
	}
	
	public void handleRemoveEdge(Node n, Node n2) {
	}

	public void handleFusion(Community resultingCommunity, Community suppressedCommunity) {
		//System.out.println("merge "+resultingCommunity.getID()+" "+suppressedCommunity.getID());
		//System.out.println(existingCommunities.get(resultingCommunity));
		//if both communities are not new, memorize this link.
		//objective : add a link from the old suppressed one to the new resulting one (but to do at the end of period in case of multiple changes)
		if(existingCommunities.keySet().contains(resultingCommunity) && existingCommunities.keySet().contains(suppressedCommunity)){
			tempMerge.add(Pair.with(suppressedCommunity, resultingCommunity));
			System.out.println("REAL FUSION !!!! "+existingCommunities.get(suppressedCommunity)+" "+existingCommunities.get(resultingCommunity));
			
			
			ArrayList<Pair<Community,Community>> toDelete = new ArrayList<>();
			ArrayList<Pair<Community,Community>> toMergeUpdate = new ArrayList<>();

			for(Pair<Community,Community> p : tempMerge){
				if(p.getValue1()==suppressedCommunity){
					toMergeUpdate.add(Pair.with(p.getValue0(), resultingCommunity));
					toDelete.add(p);
				}
			}
			tempMerge.addAll(toMergeUpdate);
			toDelete.forEach(p->tempMerge.remove(p));
			currentCommunities.remove(suppressedCommunity);
			//existingCommunities.remove(suppressedCommunity);
		}
		
		
	}
	
	
	public void finalizePeriod(){
		//write old communities and link to their previous self
				for(Community c : currentCommunities){
					if(c.getComponents().size()>=this.minimumCommunitySizeForOutput || this.alreadyDisplayedCommuniteis.contains(c)){
						alreadyDisplayedCommuniteis.add(c);
						String previousNodeID = getIDCommunity(c,true);
						String newNodeID = getIDCommunity(c,false);
	
						this.printLine(previousNodeID+separator+newNodeID);
					}
				}
				//--------------------
				
				//write merges
				for(Pair<Community,Community> p : tempMerge){
					if(p.getValue0().getComponents().size()>=this.minimumCommunitySizeForOutput || this.alreadyDisplayedCommuniteis.contains(p.getValue0())){
						if(p.getValue1().getComponents().size()>=this.minimumCommunitySizeForOutput || this.alreadyDisplayedCommuniteis.contains(p.getValue1())){
							alreadyDisplayedCommuniteis.add(p.getValue0());
							alreadyDisplayedCommuniteis.add(p.getValue1());


							if(currentCommunities.contains(p.getValue1())){
							this.printLine(getIDCommunity(p.getValue0(), true)+separator+getIDCommunity(p.getValue1(), false));
							}
							else{
								System.err.println("BE CAREFUL : merge not printed between"+getIDCommunity(p.getValue0(),true)+" and "+getIDCommunity(p.getValue1(),false));
							}
						}
					}
				}
				tempMerge.clear();
	}
	
	public void handleDateChange(String date) {
		this.previousPeriod = this.currentPeriod;
		this.currentPeriod=date;
		//System.out.println("change handled correctly");
		finalizePeriod();
		
		
		
		
		//insert new communities in the collection of old communities
		for(Community c : tempNewCommunities){
			existingCommunities.put(c,idCom++);
			currentCommunities.add(c);
		}
		tempNewCommunities = new ArrayList<>();
		
		
	}

	
	public String getIDCommunity(Community c, boolean old){
		if(old)
			return this.previousPeriod+"_"+existingCommunities.get(c);
		else
			return this.currentPeriod+"_"+existingCommunities.get(c);


	}
}
