package Outputs;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import org.javatuples.Pair;

import TemporalNetworkManipulation.Community;
import TemporalNetworkManipulation.MyDate;
import TemporalNetworkManipulation.MyDateManager;
import TemporalNetworkManipulation.Network;
import TemporalNetworkManipulation.Node;



/**this class is to print static communities. It prints only the communities alive at the end of the process.
 * the communities are printed in a way allowing overlap : each line correspond to a community. 
 * Each line contains all the nodes of the community. Standard format as used by several static algorithms
 *
 * @author remy cazabet
 *
 */
public class OutputSnapshots implements OutputHandlerInterface{

	
	private String separator = "	" ; //tab
	PrintWriter fileout=null;
	Network theNetwork;
	
	String previousPeriod;
	String currentPeriod;
	
	int idCom = 0;
	Set<Community> currentCommunities = new HashSet<>();

	HashMap<Community,Integer> existingCommunities = new HashMap<>();

	ArrayList<Community> tempNewCommunities = new ArrayList<Community>();
	
	
	int minimumCommunitySizeForOutput = 5;
	Set<Community> alreadyDisplayedCommuniteis = new HashSet<>();

	
	
	
	public OutputSnapshots(MyDateManager dateManager, Network theNetwork) {
		this.theNetwork = theNetwork;
	}

	public void initialise(String outputFile){
		
		try {
			fileout = new PrintWriter(new BufferedWriter (new FileWriter(outputFile+".snap")));
		} catch (IOException e) {
			System.out.println("problem to write in :  "+ outputFile);
			e.printStackTrace();
		}
	}
	
	public void terminate(){
		this.previousPeriod = this.currentPeriod;
		this.currentPeriod="END";
		finalizePeriod();
		fileout.close();
	}
	
	public void printLine(String line){

		fileout.println(line);
	}
	
	
	
	
	public void handleGrowth(Node n, Community c){
		
	}
	
	public void handleGrowth(Community initialCommunity, Community resultingCommunity){
		
	}

	public void handleBirth(Community c) {
		tempNewCommunities.add(c);

		
	}

	public void handleDeath(Community c) {
		tempNewCommunities.remove(c);
		currentCommunities.remove(c);
		//existingCommunities.remove(c);
		
	}

	
	public void handleContraction(Node n, Community c) {
	}
	
	public void handleNewEdge(Node n, Node n2) {
	}
	
	public void handleRemoveEdge(Node n, Node n2) {
	}

	public void handleFusion(Community resultingCommunity, Community suppressedCommunity) {
		
	}
	
	
	public void finalizePeriod(){
		
		//insert new communities in the collection of old communities
		for(Community c : tempNewCommunities){
			
			existingCommunities.put(c,idCom++);
			currentCommunities.add(c);
		}
		tempNewCommunities = new ArrayList<>();
				
				
		for(Community c : currentCommunities){
			if(c.getComponents().size()>=this.minimumCommunitySizeForOutput || this.alreadyDisplayedCommuniteis.contains(c)){
				alreadyDisplayedCommuniteis.add(c);
				String toPrint = getIDCommunity(c, true);
				for(Node n : c.getComponents()){
					toPrint+=separator+n.getName();
				}
				this.printLine(toPrint);
			}
		}
		//--------------------
	
	}
	
	public void handleDateChange(String date) {
		this.previousPeriod = this.currentPeriod;
		this.currentPeriod=date;
		//System.out.println("change handled correctly");
		finalizePeriod();
		
		
		
		
		
		
		
	}

	
	public String getIDCommunity(Community c, boolean old){
		if(old)
			return this.previousPeriod+"_"+existingCommunities.get(c);
		else
			return this.currentPeriod+"_"+existingCommunities.get(c);


	}
}
