package implementable;


import gnu.trove.map.hash.THashMap;
import gnu.trove.set.hash.THashSet;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import tools.DebugMode;

import communityDetectionPackage.ILCDMetaAlgorithm;

import TemporalNetworkManipulation.Community;
import TemporalNetworkManipulation.Node;
import TemporalNetworkManipulation.Operations.BirthCommunityOperation;
import TemporalNetworkManipulation.Operations.ContractCommunityOperation;

//RNHM stands for representative nodes and Hub minimization.
public class ImplementationRNHM extends Implementation {

	private int initialComSize;
	public static float integrateParameter;
	public static float fusionParameter;
	
	
	
	//for speed optimisation
	static THashMap<Community, Float> memorizedcohesion = new THashMap<Community, Float>();

	
	
	public ImplementationRNHM(int intialComSize, float integrateParameter, float fusionParameter){
		this.initialComSize=intialComSize;
		this.integrateParameter = integrateParameter;
		this.fusionParameter = fusionParameter;
	}
	
	@Override
	public boolean GROWTH(Node candidate, Community com) {
		//DebugMode.printDebug("------ ? integrate "+candidate.getName()+" in "+com.getID()+" "+this.getBelongingStrength(candidate, com)+" > "+this.getIntrinsicCohesion(com));

		if(this.getBelongingStrength(candidate, com) >= integrateParameter * getIntrinsicCohesion(com)){
			//OPTIMISATION
			memorizedcohesion.remove(com);
			
			return true;
		}
		
		
		
		return false;
	}

	@Override
	public ArrayList<Community> BIRTH(Node n1, Node n2) {
		ArrayList<Community> newCommunitiesToReturn = new ArrayList<Community>();
		
		if(this.initialComSize<3 || this.initialComSize>4){
			System.err.println("sorry but, currently, intial communities must have a size of 3 or 4. Contact me if questions.");
			System.exit(-1);
		}
		
		//OPTIONAL (OPTIMIZATION): we do not check for the creation of a new community if the link is created inside
		//an existing community. The reason is that this link, as it is created inside a community, is very probable
		//and therefore is not an argument to create a new community.
		
		//getting communities in common
		THashSet<Community> commonComs = new THashSet<Community> (n1.getCommunities());
		commonComs.retainAll(n2.getCommunities());
		
		if(commonComs.size()==0){
			if(this.initialComSize==3)
				this.birthCase3(newCommunitiesToReturn, commonComs,n1,n2);
			if(this.initialComSize==4)
				this.birthCase4(newCommunitiesToReturn, commonComs,n1,n2);
	
		}
		return newCommunitiesToReturn;
	}

	@Override
	public ArrayList<Community> CONTRACTION_DIVISION(Community affectedCom,Node testedNode, ILCDMetaAlgorithm ilcd) {
		//------------------------------
		//no divisions with this version
		//------------------------------

				
		ArrayList<Community> result = new ArrayList<Community>();
		result.add(affectedCom);
		
		//if the node has already been removed, no contraction
		if(!testedNode.getCommunities().contains(affectedCom)){
			return result;
		}

		
		//compute the intrinsic cohesion of the community without the node
		float adaptedIntrinsicCohesion = getIntrinsicCohesion(affectedCom)-getRepresentativity(testedNode, affectedCom);
		
		if(this.getBelongingStrength(testedNode, affectedCom) >= integrateParameter * adaptedIntrinsicCohesion){
			return result;
		}
		else{
			//OPTIMISATION
			memorizedcohesion.remove(affectedCom);
			
			//affectedCom.removeNodeFromCommunity(testedNode);
			ilcd.contract(testedNode, affectedCom);
			//Operation op = new ContractCommunityOperation(affectedCom, testedNode);
			
			//for all neighbors in the same com, check if they must be removed
			for(Node n : testedNode.getNeighborsInCommunity(affectedCom)){
					this.CONTRACTION_DIVISION(affectedCom, n, ilcd);
			}
			
			return(result);
		}
	}

	@Override
	public boolean DEATH(Community testedCom) {
		//OPTIMISATION
		memorizedcohesion.remove(testedCom);
		
		return testedCom.getComponents().size()<this.initialComSize ;
			
	}


	@Override
	public ArrayList<Community> FUSION(Community toBeAbsorbed, Community toAbsorb, ILCDMetaAlgorithm ilcd) {
		
		
		ArrayList<Community> result = new ArrayList<Community>();
		result.add(toBeAbsorbed);
		result.add(toAbsorb);
		
		
		ArrayList<Node> commonNodes = new ArrayList<Node>(toBeAbsorbed.getComponents());
		commonNodes.retainAll(toAbsorb.getComponents());
		
		float representativityOfCommonNodes = 0;
		for(Node n : commonNodes){
			representativityOfCommonNodes+=getRepresentativity(n, toBeAbsorbed);
		}
		
		//if the fusion must be done
		if(representativityOfCommonNodes>getIntrinsicCohesion(toBeAbsorbed)*this.fusionParameter){
			
			//OPTIMISATION
			memorizedcohesion.remove(toBeAbsorbed);
			memorizedcohesion.remove(toAbsorb);
			
			//remove the youngest community
			Community youngest = (result.get(0).youngerThan(result.get(1))) ? result.get(0) : result.get(1);

			////result.remove(toBeAbsorbed);
			//do the fusion by updating the community to keep (not necessarily the oldest ! be careful)
			ArrayList<Node> mightIntegrate = new ArrayList<Node>(toBeAbsorbed.getComponents());
			mightIntegrate.removeAll(commonNodes);
			for(Node n : mightIntegrate){
				if(this.GROWTH(n, toAbsorb))
					ilcd.addNodeToCommunity(n, toAbsorb);
			}
			
			//update the oldest community to make it match the youngets one.
			//as a summary, the community has been created by adding some nodes of the oldest to the youngest
			//but for community tracking reasons, we say that the resulting community is in fact the oldest
			if(youngest!=toBeAbsorbed){
				Community oldest = toBeAbsorbed;
				Set<Node> toRemove = new THashSet<>(oldest.getComponents());
				toRemove.removeAll(youngest.getComponents());
				for(Node n : toRemove){
					ilcd.contract(n, oldest);
				}
				
				Set<Node> toAdd = new THashSet<>(youngest.getComponents());
				toAdd.removeAll(oldest.getComponents());
				for(Node n : toAdd){
					ilcd.addNodeToCommunity(n, oldest);
				}
			}
			//if the community to keep is not the right one, update
			result.remove(youngest);

		}
		return result;
		
	}
	
	public static float getRepresentativity(Node n, Community c){
		
		//String idReltion = getIdRelation(n, c);
		
		THashSet<Node> neighborsInC = new THashSet<Node>(n.getNeighbors());
		float nbNeighbors = neighborsInC.size();
		neighborsInC.retainAll(c.getComponents());
		float nbNeighborsInC = neighborsInC.size();
		
		return nbNeighborsInC / nbNeighbors ;
		
	}
	
	//private static String getIdRelation(Node n, Community c){
		//return c.getID()+n.getName();
	//}
	
	private float getBelongingStrength(Node n, Community c){
		
		THashSet<Node> neighborsInC = new THashSet<Node>(c.getComponents());
		neighborsInC.retainAll(n.getNeighbors());

		
		//will probably need an optimization for not computing again values already computed
		float belongingStrength = 0;
		if(neighborsInC.size()<2){
			return 0;
		}
		else{
			for(Node neighb : neighborsInC){
				belongingStrength+=getRepresentativity(neighb, c);
			}
		}
		
		return belongingStrength ;
		
	}
	
	public static float getIntrinsicCohesion(Community c){
		if(memorizedcohesion.containsKey(c))
			return memorizedcohesion.get(c);
		
		//will probably need an optimization for not computing again values already computed
				float intrinsicCohesion = 0;
				for(Node component : c.getComponents()){
					intrinsicCohesion+=getRepresentativity(component, c);
				}
				
				memorizedcohesion.put(c, intrinsicCohesion);
				return intrinsicCohesion ;
	}
	
	//more efficient
	/*public THashSet<Community> getCommunitiesForWhichIsRepresentative(Node n){
		THashSet<Community> result = new THashSet<Community>();
		for(Community c : n.getCommunities()){
			if(this.getRepresentativity(n, c)>this.getIntrinsicCohesion(c)/(float)c.getComponents().size()){
				result.add(c);
			}
		}
		return result;
	}*/
	
private ArrayList<Community> birthCase3(ArrayList<Community> newCommunitiesToReturn, THashSet<Community> commonComs, Node n1, Node n2){
	
	//getting neighbors in common
	THashSet<Node> commonNeighbors = new THashSet<Node>(n1.getNeighbors());
	commonNeighbors.retainAll(n2.getNeighbors());

	for(Node candidate : commonNeighbors){
		//THashSet<Community> commonComsAllNodes= new THashSet<Community>(candidate.getCommunities());
		//commonComsAllNodes.retainAll(commonComs);
		//if and only if these nodes are not already in the same community
		//if(commonComsAllNodes.size()==0){
			//HashSet<Node> initialNodes = new HashSet<Node>();
			//initialNodes.add(n1);
			//initialNodes.add(n2);
			//initialNodes.add(candidate);
			//Operation op = new BirthCommunityOperation();
			
		
			
			Community newCom = new Community();
			newCom.addNodeToCommunity(n1);
			newCom.addNodeToCommunity(n2);
			newCom.addNodeToCommunity(candidate);
			newCommunitiesToReturn.add(newCom);
		//}
	}
	
	return newCommunitiesToReturn;
	}

private ArrayList<Community> birthCase4(ArrayList<Community> newCommunitiesToReturn, THashSet<Community> commonComs, Node n1, Node n2){
	
	//getting neighbors in common
		ArrayList<Node> commonNeighbors = new ArrayList<Node>(n1.getNeighbors());
		commonNeighbors.retainAll(n2.getNeighbors());
		
		for(int i=0;i<commonNeighbors.size();i++){
			for(int j=i;j<commonNeighbors.size();j++){
				if(commonNeighbors.get(i).getNeighbors().contains(commonNeighbors.get(j))){
					Community newCom = new Community();
					newCom.addNodeToCommunity(n1);
					newCom.addNodeToCommunity(n2);
					newCom.addNodeToCommunity(commonNeighbors.get(i));
					newCom.addNodeToCommunity(commonNeighbors.get(j));

					newCommunitiesToReturn.add(newCom);
				}
			}
		}
		
	
	return newCommunitiesToReturn;
		
	}
}
